<?php

/**
 * @file
 *
 * Filter to rewrite facet items via a dedicated hook
 *
 * TODOs
 * - determine whether the definition and invocation of hook_facet_items_alter
 *   shouldn't happen within FacetAPI directly.
 *   Advantage of this module: The cost of invocation occurs only it needed.
 */

/**
 * Facetapi plugin for rewriting facet items by
 * implementing a hook_facet_items_alter(&$build, &$settings);
 */
class FacetapiFilterEAVfield extends FacetapiFilter {

  /**
   * Filters facet items.
   */
  public function execute(array $build) {

    $settings = $this->settings->settings;
    if (!(isset($settings['eavfield_filter_items']) && $settings['eavfield_filter_items'])) {
      return $build;
    }

    if (isset($settings['eavfield_type'])) {
      switch ($settings['eavfield_type']) {
        case 'list':
          foreach ($build as $key => $item) {
            if (isset($item['#indexed_value']) && is_numeric($item['#indexed_value'])) {
              $itemlist = eavfield_itemlist_load($item['#indexed_value']);
              if (isset($itemlist->name)) {
                $build[$key]['#markup'] = check_plain($itemlist->name);
              }
            }
          }
          break;
      }
    }
    return $build;
  }

  /**
   * Adds hook documentation
   */
  function settingsForm(&$form, &$form_state) {
    $settings = $this->settings;
    $form['eavfield_filter_items'] = array(
      '#title' => t('EAVfield filter'),
      '#type' => 'checkbox',
      '#default_value' => isset($settings->settings['eavfield_filter_items']) ? $settings->settings['eavfield_filter_items'] : FALSE,
    );

    $form['eavfield_type'] = array(
      '#title' => t('Filter type'),
      '#type' => 'select',
      '#options' => array(
        'none' => t('- Select value -'),
        'list' => t('Attribute ID to name a replacement'),
      ),
      '#default_value' => isset($settings->settings['eavfield_type']) ? $settings->settings['eavfield_type'] : 'none',
    );

  }

}
