<?php

/**
 * @file
 * Contains SearchApiAlterAddAggregation.
 */

/**
 * Search API data alteration callback that adds an URL field for all items.
 */
class SearchApiEAVfieldIntegration extends SearchApiAbstractAlterCallback {

  /**
   * The type of aggregation currently performed.
   *
   * Used to temporarily store the current aggregation type for use of
   * SearchApiAlterAddAggregation::reduce() with array_reduce().
   *
   * @var string
   */
  protected $reductionType;

  public function configurationForm() {
    //$form['#attached']['css'][] = drupal_get_path('module', 'search_api') . '/search_api.admin.css';

    //Загружаем список доступных атрибутов
    $attributes = eavfield_attributes_list();

    //Загружаем список полей с типом 'eavfield'
    $eavfields = array();
    $fields_info = field_info_fields();
    foreach ($fields_info as $f) {
      if ($f['type'] == 'eavfield') {
        $eavfields[$f['field_name']] = $f['field_name'];
      }
    }

    //Загружаем спсиок ранее созданных полей для индексации (если они есть)
    $additional = empty($this->options['fields']) ? array() : $this->options['fields'];

    //Загружаем список доступных вариантов индексации (ключ или значение поля)
    $types = $this->getTypes();

    $form['#id'] = 'edit-callbacks-search-api-alter-add-eavfield-settings';
    $form['description'] = array(
      '#markup' => t('<p>This data alteration lets you define additional fields that will be added to this index. </p>'),
    );
    $form['fields']['#prefix'] = '<div id="search-api-alter-add-eavfield-field-settings">';
    $form['fields']['#suffix'] = '</div>';
    if (isset($this->changes)) {
      $form['fields']['#prefix'] .= '<div class="messages warning">All changes in the form will not be saved until the <em>Save configuration</em> button at the form bottom is clicked.</div>';
    }
    foreach ($additional as $name => $field) {
      $form['fields'][$name] = array(
        '#type' => 'fieldset',
        '#title' => $field['name'] ? $field['name'] : t('New attribute'),
        '#collapsible' => TRUE,
        '#collapsed' => (boolean) $field['name'],
      );
      $form['fields'][$name]['name'] = array(
        '#type' => 'textfield',
        '#title' => t('New field name'),
        '#default_value' => $field['name'],
        '#required' => TRUE,
      );
      $form['fields'][$name]['field_name'] = array(
        '#type' => 'select',
        '#title' => t('Field name'),
        '#options' => $eavfields,
        '#default_value' => $field['field_name'],
        '#required' => TRUE,
      );
      $form['fields'][$name]['attr'] = array(
        '#type' => 'select',
        '#title' => t('Attribute'),
        '#options' => $attributes,
        '#default_value' => $field['attr'],
        '#required' => TRUE,
      );
      $form['fields'][$name]['type'] = array(
        '#type' => 'select',
        '#title' => t('Type'),
        '#options' => $types,
        '#default_value' => $field['type'],
        '#required' => TRUE,
      );
      $form['fields'][$name]['actions'] = array(
        '#type' => 'actions',
        'remove' => array(
          '#type' => 'submit',
          '#value' => t('Remove field'),
          '#submit' => array('_search_api_add_eavfield_field_submit'),
          '#limit_validation_errors' => array(),
          '#name' => 'search_api_add_eavfield_remove_' . $name,
          '#ajax' => array(
            'callback' => '_search_api_add_eavfield_field_ajax',
            'wrapper' => 'search-api-alter-add-eavfield-field-settings',
          ),
        ),
      );
    }
    $form['actions']['#type'] = 'actions';
    $form['actions']['add_field'] = array(
      '#type' => 'submit',
      '#value' => t('Add new attribute'),
      '#submit' => array('_search_api_add_eavfield_field_submit'),
      '#limit_validation_errors' => array(),
      '#ajax' => array(
        'callback' => '_search_api_add_eavfield_field_ajax',
        'wrapper' => 'search-api-alter-add-eavfield-field-settings',
      ),
    );
    return $form;
  }

  public function configurationFormValidate(array $form, array &$values, array &$form_state) {
    unset($values['actions']);
    if (empty($values['fields'])) {
      return;
    }

    foreach ($values['fields'] as $name => $field) {
      // $aid = $field['attr'];
      // $values['fields'][$name]['attributes'] = eavfield_attributes_load($aid);
    }

    /*
    foreach ($values['fields'] as $name => $field) {
      $attr = $values['fields'][$name]['fields'] = array_values(array_filter($field['fields']));
      unset($values['fields'][$name]['actions']);
      if ($field['name'] && !$fields) {
        form_error($form['fields'][$name]['fields'], t('You have to select at least one field to aggregate. If you want to remove an aggregated field, please delete its name.'));
      }
    }
    */
  }

  public function configurationFormSubmit(array $form, array &$values, array &$form_state) {
    if (empty($values['fields'])) {
      return array();
    }
    //$index_fields = $this->index->getFields(FALSE);
    foreach ($values['fields'] as $name => $field) {
      if (!$field['name']) {
        unset($values['fields'][$name]);
      }
      else {
        $values['fields'][$name]['description'] = $this->fieldDescription($field);
      }
    }
    $this->options = $values;
    return $values;
  }

  public function alterItems(array &$items) {
    if (!$items) {
      return;
    }

    if (isset($this->options['fields'])) {
      foreach ($items as $item) {
        foreach ($this->options['fields'] as $name => $field) {
          //устанавливаем значение по умолчанию
          $item->$name = NULL;
          $values = array();
          //проверяем, есть ли у текущего объекта указанное поле
          if (isset($item->$field['field_name'])) {
            //Обработка node
            if (isset($item->nid) && !empty($item->nid)) {
              $field_values = field_get_items('node', $item, $field['field_name']);

              if (isset($field_values[0]['aid'])) {
                foreach ($field_values as $field_value) {
                  if ($field_value['aid'] == $field['attr']) {
                    $values[] = $this->getIndexValue($field_value['value'], $field['attr'], $field['type']);;
                  }
                }
              }
            }
          }

          if (count($values)) {
            $item->$name = $values[0];
          }
        }
      }
    }
   // dpm($item);
  }

  public function propertyInfo() {
    $ret = array();
    if (isset($this->options['fields'])) {
      foreach ($this->options['fields'] as $name => $field) {
        $type = $this->getIndexType($field);
        $ret[$name] = array(
          'label' => $field['name'],
          'description' => empty($field['description']) ? '' : $field['description'],
          'type' => $type,
        );
      }
    }
    return $ret;
  }

  /**
   * Helper method for creating a field description.
   */
  protected function fieldDescription(array $field) {
    $desc = t('AEV attribute: ');

    $fields = array();
    if (isset($field['attr'])) {
      $attr = eavfield_attributes_load($field['attr']);
      if ($attr) {
        $desc .= '"' . $attr->name . ':' . $field['type'] . '"';
      }
    }
    return $desc;
  }

  /**
   * Возвращает тип поля для индекса в зависимости от текщего атрибута и опции его вывода (key или value)
   * @param string $info
   * @param bool $attr
   */
  protected function getIndexType($field) {
    $attr = eavfield_attributes_load($field['attr']);
    $type = $field['type'];
    if ($attr->type == 'bool') {
      return 'boolean';
    }

    if ($attr->type == 'list' && $type == 'value') {
      //entity ?
      return 'text';
    }

    if ($type == 'source') {
      return 'integer';
    }
    else {
      if ($attr->type == 'int') {
        return 'integer';
      }
    }
    return 'text';
  }

  /**
   * Возвращает значение для индекса в зависимости от текщего атрибута и опции его вывода (key или value)
   *
   *
   */
  protected function getIndexValue($value, $attr, $type) {
    if (is_numeric($attr)) {
      $attr = eavfield_attributes_load($attr);
    }

    if ($attr->disabled) {
      return NULL;
    }

    if ($type == 'source') {
      return $value;
    }
    else {
      if ($attr) {
        $value_new = eavfield_value_to_attr_return($value, $attr);
        if ($value_new !== FALSE) {
          return $value_new;
        }
      }
    }
    return NULL;
  }

  /**
   * Helper method for getting all available aggregation types.
   *
   * @param string $info
   *   (optional) One of "name", "type" or "description", to indicate what
   *   information should be returned for the types.
   *
   * @return string[]
   *   An associative array of aggregation type identifiers mapped to their
   *   names, data types or descriptions, as requested.
   */
  protected function getTypes($info = 'name') {
    switch ($info) {
      case 'name':
        return array(
          'source' => t('Source'),
          'value' => t('Value'),
        );
      case 'type':
        return array(
          'source' => 'integer',
          'value' => 'text',
        );
      case 'description':
        return array(
          'source' => t('Source (listID or Integer value)'),
          'value' => t('Text'),
        );
    }
    return array();
  }

  /**
   * Submit helper callback for buttons in the callback's configuration form.
   */
  public function formButtonSubmit(array $form, array &$form_state) {
    $button_name = $form_state['triggering_element']['#name'];
    if ($button_name == 'op') {
      // Increment $i until the corresponding field is not set, then create the
      // field with that number as suffix.
      for ($i = 1; isset($this->options['fields']['search_api_eavfield_' . $i]); ++$i) {
      }
      $this->options['fields']['search_api_eavfield_' . $i] = array(
        'name' => '',
        'type' => 'text',
        'field_name' => '',
        'attr' => array(),
      );
    }
    else {
      $field = substr($button_name, 31);
      unset($this->options['fields'][$field]);
    }
    $form_state['rebuild'] = TRUE;
    $this->changes = TRUE;
  }
}

/**
 * Submit function for buttons in the callback's configuration form.
 */
function _search_api_add_eavfield_field_submit(array $form, array &$form_state) {
  $form_state['callbacks']['search_api_alter_add_eavfield']->formButtonSubmit($form, $form_state);
}

/**
 * AJAX submit function for buttons in the callback's configuration form.
 */
function _search_api_add_eavfield_field_ajax(array $form, array &$form_state) {
  return $form['callbacks']['settings']['search_api_alter_add_eavfield']['fields'];
}



